@@ -87,7 +87,8 @@ def test_context():
 
 def test_surface():
     # TypeError: The Surface type cannot be instantiated
-    pytest.raises(TypeError, "s = cairo.Surface()")
+    with pytest.raises(TypeError):
+        cairo.Surface()
 
     f, w, h = cairo.FORMAT_ARGB32, 100, 100
     s = cairo.ImageSurface(f, w, h)
@@ -158,8 +159,9 @@ def test_image_surface_get_data():
 def test_surface_file_obj_error():
     class Fail(object):
 
-        def write(*args):
-            raise IOError
+        def write(obj, data):
+            if data:
+                raise IOError
 
     cairo.PDFSurface(Fail(), 100, 100)
     cairo.PSSurface(Fail(), 100, 100)
@@ -277,12 +279,8 @@ def test_constants():
 @pytest.mark.skipif(not hasattr(sys, "getrefcount"), reason="PyPy")
 def test_surface_get_set_mime_data_references():
     surface = cairo.ImageSurface(cairo.FORMAT_RGB24, 1, 1)
-    if sys.version_info[0] == 2:
-        v = buffer(b"bla")
-        x = buffer(v, 0, 1)
-    else:
-        v = memoryview(b"bla")
-        x = v[:1]
+    v = memoryview(b"bla")
+    x = v[:1]
     assert sys.getrefcount(v) == 2
     assert sys.getrefcount(x) == 2
     surface.set_mime_data("foo", v)
